#!/usr/bin/env python
############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import socket, sys, sha, time, urlparse
from ConfigParser import ConfigParser

import ZSI.TC
from ZSI.wstools.Namespaces import WSU

import pyGridWare.generated.stubs.Counter_services as COUNTER
from pyGridWare.generated.types.Addressing_types import *
from pyGridWare.notification.NotificationConsumer import NotificationConsumer

HOSTNAME = socket.gethostbyaddr(socket.gethostname())[0]

class QualifiedName(tuple):
    """Use to package data for QName typecode, this
    way the data is not recognized as a sequence type.
    """
    def __init__(self, value):
        tuple.__init__(self,value)


"""
Counter
    add
    createCounter
"""
def create(port):
    request = COUNTER.CreateCounterRequest()
    msg = port.createCounter(request)
    print "----- create ----- "
    print "\tAddress -- ", msg._EndpointReference._Address
    print "\tRefProp -- ", msg._EndpointReference._ReferenceProperties
    print "\t\tany -- ", msg._EndpointReference._ReferenceProperties._any
    return msg

def add(port, value=999):
    print "----- add ----- "
    request = value
    msg = port.add(request)
    print "RESPONSE: ", msg

"""
ResourceLifetime
    Destroy
    GetCurrentMessage
    SetTerminationTime
"""
def Destroy(port):
    print "----- destroy ----- "
    request = COUNTER.DestroyRequest()
    msg = port.Destroy(request)
    print "RESPONSE: ", msg

def SetTerminationTime(port, interval):
    print "----- SetTerminationTime ----- "
    request = COUNTER.SetTerminationTimeRequest()
    request._RequestedTerminationTime = time.gmtime(time.time()+interval)
    msg = port.SetTerminationTime(request)
    print "TerminationTime: ", msg._NewTerminationTime
    print "CurrrentTime:    ", msg._CurrentTime

"""
ResourceProperties
    GetResourceProperty
"""
def GetResourceProperty(port):
    print "----- GetResourceProperty ----- "
    request = QualifiedName(("http://counter.com", "Value"))
    msg = port.GetResourceProperty(request)
    print "RESPONSE: ", msg._any



def counter(url):
    path = urlparse.urlparse(url)[2]
    kw = {'tracefile':sys.stdout, 'url':path}

    scheme,netloc,path,params,query,fragment = urlparse.urlparse(url)
    locator = COUNTER.CounterServiceLocator()
    port = locator.getCounterPortType(portAddress=url, **kw)

    response = create(port)
    port.endPointReference = response._EndpointReference

    add(port, 11)
    GetResourceProperty(port)
    add(port, 13)
    #SetTerminationTime(port, 5.0)
    Destroy(port)
    #add(port, 13)
    #time.sleep(10)
    add(port, 11)

 
def main():
    CONFIG_FILE = 'config.txt'
    SEC_PATHS = 'Paths'
    TEST = 'CounterService'

    cp = ConfigParser()
    cp.read(CONFIG_FILE)

    path = cp.get(SEC_PATHS, 'AuthService')
    netloc = '%s:%d'%('localhost', 8080)
    urlTuple = ('http',netloc,path,'','','')
    url = urlparse.urlunparse(urlTuple)

    path = cp.get(SEC_PATHS, 'CounterService')
    urlTuple = ('http',netloc,path,'','','')
    url = urlparse.urlunparse(urlTuple)
    counter(url)


if __name__ == '__main__':
    main()

