############################################################################
# Joshua R. Boverhof, LBNL
# See Copyright for copyright notice!
###########################################################################
from ZSI.ServiceContainer import WSResourceService
from pyGridWare.generated.stubs.Authenticator_services import *
from pyGridWare.security.gss.GssSecurityContext import GssSecurityContext
from pyGridWare.security.gss.GssSecureConversation import GssSecureConversation


class Authenticator(WSResourceService):
    soapAction = {
        'http://wsrf.globus.org/security/authentication/secureconv/SecureContextEstablishmentPortType/initTokenExchange':'soap_initTokenExchange',
        'http://wsrf.globus.org/security/authentication/secureconv/SecureContextEstablishmentPortType/continueTokenExchangeRequest': 'soap_continueTokenExchange',
        }
    wsAction = {
        'soap_initTokenExchange':'http://wsrf.globus.org/security/authentication/secureconv/SecureContextEstablishmentPortType/initTokenExchangeResponse',
        'soap_continueTokenExchange':'http://wsrf.globus.org/security/authentication/secureconv/SecureContextEstablishmentPortType/continueTokenExchangeResponse',
        }

    def __init__(self, post='/wsrf/services/', **kw):
        WSResourceService.__init__(self, post)

    def soap_continueTokenExchange(self, ps, address):
        # input vals in request object
        args = ps.Parse( ContinueTokenExchangeRequest() )

        # assign return values to response object
        response = ContinueTokenExchangeResponse()
        GssSecureConversation.continueTokenExchange(args, response)

        return response

    def soap_initTokenExchange(self, ps, address):
        # input vals in request object
        args = ps.Parse( InitTokenExchangeRequest() )

        # assign return values to response object
        response = InitTokenExchangeResponse()
        GssSecureConversation.initTokenExchange(args, response)

        return response

"""
class SecureConversation:
    MECHANISM = "http://www.globus.org/2002/04/xmlenc#gssapi-gsi"

    def initTokenExchange(cls, request, response):
        key = ("http://wsrf.globus.org/security/authentication/secureConv/types","mechanism-type")
        mech = request._initContextToken._attrs[key]
        if mech != cls.MECHANISM:
            raise RuntimeError, 'Bad Mechanism(%s)' %mech
        base64Token = request._initContextToken._base64Token
        context_id = request._initContextToken._context_id
        context = GssSecurityContext()

        home = GssSecurityContext.getResourceHome()
        home.create(context, context_id)

        response._contextTokenOut = ContextToken()
        response._contextTokenOut.setBase64Token(context.acceptSecurityContext(base64Token))
        response._contextTokenOut.setContextID(context_id)
        response._contextTokenOut.setAttributeContinueNeeded(1)
    initTokenExchange = classmethod(initTokenExchange)

    def continueTokenExchange(cls, request, response):
        base64Token = request._contextToken._base64Token
        context_id = request._contextToken._context_id

        home = GssSecurityContext.getResourceHome()
        context = home.get(context_id)

        response._contextTokenOut = ContextToken()
        token = context.acceptSecurityContext(base64Token)
        response._contextTokenOut.setBase64Token(token)
        if token:
            response._contextTokenOut.setAttributeContinueNeeded(1)
        else:
            response._contextTokenOut.setAttributeContinueNeeded(0)
        response._contextTokenOut.setContextID(context_id)
    continueTokenExchange = classmethod(continueTokenExchange)



class ContextToken:
    def __init__(self):
        self._base64Token = None
        self._context_id = None
        self._attrs = {}

    def setBase64Token(self, base64Token):
        self._base64Token = base64Token
        
    def setContextID(self, context_id):
        self._context_id = context_id

    def setAttributeContinueNeeded(self, continue_needed):
        key = ("http://wsrf.globus.org/security/authentication/secureConv/types","continue-needed")
        self._attrs[key] = continue_needed


"""
