import time
import ZSI.TC
from ZSI.ServiceContainer import WSResourceService
from ZSI.wstools.Namespaces import WSU

from pyGridWare.generated.stubs.Counter_services import \
    AddInputMessage, AddOutputMessage, CreateCounterRequest, CreateCounterResponse,\
    DestroyRequest, DestroyResponse, GetCurrentMessageRequest, GetCurrentMessageResponse,\
    GetResourcePropertyRequest, GetResourcePropertyResponse, SetTerminationTimeRequest,\
    SetTerminationTimeResponse, SubscribeRequest, SubscribeResponse
from pyGridWare.generated.types.Counter_services_types import counter_com
from pyGridWare.generated.properties.CounterResourceProperties import \
    CounterKey,CounterProperties

from pyGridWare.generated.properties.SubscriptionResourceProperties import \
    SubscriptionKey

from pyGridWare.addressing.AddressingUtils import AddressingUtils
from pyGridWare.lifetime.ResourceLifetime import ImmediateResourceTermination,\
    ScheduledResourceTermination
from pyGridWare.resource.ResourceHome import PersistentResourceHome
from pyGridWare.resource.ResourceContext import ResourceContext
from pyGridWare.security.gss.GssService import GssSecureResourceService


class AddValue(int):
    '''Encapsulate data and typecode for an int
    '''
    typecode = AddOutputMessage()


class ReferencePropertiesContainer:
    '''Container for Reference Data 
    '''
    class ReferenceProperties:
        def __init__(self):
            self._any = None
        def setResourceID(self, key):
            self._any = key

    def setAddress(self, address):
        self._Address = address

    def setResourceID(self, key):
        self._ReferenceProperties = ReferencePropertiesContainer.ReferenceProperties()
        self._ReferenceProperties.setResourceID(key)


class CounterHome(PersistentResourceHome):

    def getResourceContext(self, ps, address):
        key = AddressingUtils.getResourceID(ps, CounterKey.typecode)
        return self.get(key)


class CounterContext(ResourceContext):
    home = CounterHome()

    def __init__(self):
        ResourceContext.__init__(self)
        self.resourceID = None
        self.properties = CounterProperties()
        self.terminationTime = None
        self.url = None

    def setResourceID(self, resourceID):
        self.resourceID = CounterKey(resourceID)


#class Counter(WSResourceService):
class CounterInterface:
    soapAction = {
        'http://www.ibm.com/xmlns/stdwip/web-services/WS-ResourceLifetime/Destroy': 'wsa_Destroy',
        'http://www.ibm.com/xmlns/stdwip/web-services/WS-BaseNotification/GetCurrentMessage': 'wsa_GetCurrentMessage',
        'http://www.ibm.com/xmlns/stdwip/web-services/WS-ResourceProperties/GetResourceProperty': 'wsa_GetResourceProperty',
        'http://www.ibm.com/xmlns/stdwip/web-services/WS-ResourceLifetime/SetTerminationTime': 'wsa_SetTerminationTime',
        'http://www.ibm.com/xmlns/stdwip/web-services/WS-BaseNotification/Subscribe': 'wsa_Subscribe',
        'http://counter.com/CounterPortType/addRequest': 'wsa_add',
        'http://counter.com/CounterPortType/createCounter': 'wsa_createCounter',
        'http://counter.com/CounterPortType/createCounterRequest':'wsa_createCounter',
        'http://counter.com/CounterPortType/DestroyRequest': 'wsa_Destroy',
        'http://counter.com/CounterPortType/SetTerminationTimeRequest': 'wsa_SetTerminationTime',
        'http://counter.com/CounterPortType/GetResourcePropertyRequest':'wsa_GetResourceProperty',
        'http://counter.com/CounterPortType/SubscribeRequest':'wsa_Subscribe',
        }
    wsAction = {
       'wsa_add':'http://counter.com/CounterPortType/addResponse',
       'wsa_createCounter':'http://counter.com/CounterPortType/createCounterResponse',
       'wsa_Destroy':'http://counter.com/CounterPortType/DestroyResponse',
       'wsa_SetTerminationTime':'http://counter.com/CounterPortType/SetTerminationTimeResponse',
       'wsa_GetResourceProperty':'http://counter.com/CounterPortType/GetResourcePropertyResponse',
       'wsa_Subscribe':'http://counter.com/CounterPortType/SubscribeResponse',
    }


    def wsa_Destroy(self, ps, address):
        # input vals in request object
        args = ps.Parse( DestroyRequest() )

        home = CounterContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)
        ImmediateResourceTermination.Destroy(ctx)

        # assign return values to response object
        response = DestroyResponse()

        return response

    def wsa_GetCurrentMessage(self, ps, address):
        # input vals in request object
        args = ps.Parse( GetCurrentMessageRequest() )

        # assign return values to response object
        response = GetCurrentMessageResponse()

        return response

    def wsa_GetResourceProperty(self, ps, address):
        # input vals in request object
        args = ps.Parse( GetResourcePropertyRequest() )

        home = CounterContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)
        properties = ctx.getResourceProperties()

        namespaceURI,element = args
        property = properties.getResourceProperty(namespaceURI,element)

        # assign return values to response object
        response = GetResourcePropertyResponse()
        #key = (namespaceURI,element)
        response._any = property

        return response

    def wsa_SetTerminationTime(self, ps, address):
        # input vals in request object
        args = ps.Parse( SetTerminationTimeRequest() )

        home = CounterContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)
        ScheduledResourceTermination.SetTerminationTime(ctx, args._RequestedTerminationTime)

        # assign return values to response object
        response = SetTerminationTimeResponse()
        response._NewTerminationTime = time.gmtime(ctx.getTerminationTime())
        response._CurrentTime = time.gmtime(time.time())
        return response

    def wsa_Subscribe(self, ps, address):
        # input vals in request object
        args = ps.Parse( SubscribeRequest() )

        # Pass SubscriptionRequest off
        # ...

        # assign return values to response object
        url = self.getServiceURL()
        response = SubscribeResponse()
        response._SubscriptionReference = ReferencePropertiesContainer()
        #response._SubscriptionReference.setAddress("http://192.168.1.3:8080/wsrf/services/SubscriptionManagerService")
        response._SubscriptionReference.setAddress(url)
        response._SubscriptionReference.setResourceID(SubscriptionKey(3000))

        return response

    def wsa_add(self, ps, address):
        args = ps.Parse( AddInputMessage() )
        home = CounterContext.getResourceHome()
        ctx = home.getResourceContext(ps, address)
        counter = ctx.getResourceProperties()
        result = counter.getValue()
        result += args
        counter.setValue(result)

        response = AddValue(result)

        return response

    def wsa_createCounter(self, ps, address):
        args = ps.Parse( CreateCounterRequest() )

        ctx = CounterContext()
        url = self.getServiceURL()
        ctx.setAddress(url)
        home = ctx.getResourceHome()
        key = home.create()
        home.add(ctx, key)
        ctx.setResourceID(key)
        epr = AddressingUtils.createEndpointReference(ctx)

        response = CreateCounterResponse()
        response._EndpointReference = epr

        return response


class Counter(WSResourceService, CounterInterface):
    soapAction = CounterInterface.soapAction
    wsAction = CounterInterface.wsAction

    def __init__(self, post='/wsrf/services/CounterService', **kw):
        WSResourceService.__init__(self, post)


class SecureCounter(GssSecureResourceService, CounterInterface):
    soapAction = CounterInterface.soapAction
    wsAction = CounterInterface.wsAction

    def __init__(self, post='/wsrf/services/SecureCounterService', **kw):
        GssSecureResourceService.__init__(self, post)

    def wsa_Destroy(self, ps, address):
        response = CounterInterface.wsa_Destroy(self, ps, address)

        # Specify Signature over response if security specified.
        response.typecode.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
        response._attrs = {}
        response._attrs[(WSU.BASE,"Id")] = "digestSource"

        return response

    def wsa_GetResourceProperty(self, ps, address):
        response = CounterInterface.wsa_GetResourceProperty(self, ps, address)

        # Specify Signature over response if security specified.
        response.typecode.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
        response._attrs = {}
        response._attrs[(WSU.BASE,"Id")] = "digestSource"

        return response

    def wsa_add(self, ps, address):
        response = CounterInterface.wsa_add(self, ps, address)

        # Specify Signature over response if security specified.
        response.typecode.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
        response._attrs = {}
        response._attrs[(WSU.BASE,"Id")] = "digestSource"

        return response

    def wsa_createCounterInterface(self, ps, address):
        response = CounterInterface.wsa_createCounter(self, ps, address)

        # Specify Signature over response if security specified.
        #counter_com.createCounterInterface.esponse_Dec.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
        response.typecode.attribute_typecode_dict = {(WSU.BASE,"Id"):ZSI.TC.String()}
        response._attrs = {}
        response._attrs[(WSU.BASE,"Id")] = "digestSource"

        return response
        



