############################################################################
# Joshua R. Boverhof, LBNL
# See LBNLCopyright for copyright notice!
###########################################################################
import sha, base64
from Ft.Xml import XPath
import ZSI.TC
from ZSI.wstools.Namespaces import WSU, DSIG, SOAP
from ZSI.utility import ElementProxy
import pyGridWare.generated.stubs.Authenticator_services as AUTH
from pyGridWare.security.gss.GssSecureConversation import GssSecurityContext


AUTH.schemas_xmlsoap_org_ws_2002_04_secext.Security_Dec.attribute_typecode_dict = {(SOAP.ENV,'actor'):ZSI.TC.String()}
class Holder: pass
class Signature:
    typecode = AUTH.www_w3_org_2000_09_xmldsig.Signature_Dec()


class GssSignatureHandler:
    """
<wsse:Security soapenv:actor="" soapenv:mustUnderstand="0" xmlns:wsse="http://schemas.xmlsoap.org/ws/2002/04/secext" xmlns:wsu="http://schemas.xmlsoap.org/ws/2002/04/utility">
  <ds:Signature xmlns:ds="http://www.w3.org/2000/09/xmldsig#">

    <ds:SignedInfo>
      <ds:CanonicalizationMethod Algorithm="http://www.w3.org/TR/2001/REC-xml-c14n-20010315"/>
      <ds:SignatureMethod Algorithm="http://www.globus.org/2002/04/xmlenc#gssapi-sign"/>
      <ds:Reference URI="#digestSource">
        <ds:DigestMethod Algorithm="http://www.w3.org/2000/09/xmldsig#sha1"/>
        <ds:DigestValue>l1flSEDSfkc0THuJP6mV0GsHWwU=</ds:DigestValue>
      </ds:Reference>
    </ds:SignedInfo>

    <ds:SignatureValue>AAAAAAAAAAIAAAM/O2xmRW20i5bhOdmpdDvkEiDHvag=</ds:SignatureValue>

    <ds:KeyInfo>
      <ds:KeyName>00000000-0098-df1f-00000000-000000d009d5</ds:KeyName>
    </ds:KeyInfo>

  </ds:Signature>
</wsse:Security>
    """
    def __init__(self):
        self._can_algo = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315"
        self._sig_algo = "http://www.globus.org/2002/04/xmlenc#gssapi-sign"
        self._dig_algo = "http://www.w3.org/2000/09/xmldsig#sha1"
        self._contextID = None

    def setContextID(self, KeyName):
        self._contextID =  KeyName

    def getContextID(self):
        return self._contextID

    def verify(self, ps):
        """
        The algorithm to verify:

        Get and canonicalize the signedInfo node

        Get the value of the SignatureValue node

        base64 decode the Signature Value

        pass the canonicalized signed info node and the base64 decoded signature to
        verify mic
        """
        
        element = ElementProxy(None, message=ps.header)
        idExp = XPath.Compile('//*[@wsu:Id]')
        nodes = element.evaluate(expression=idExp, processorNss={'wsu':WSU.BASE})
        if len(nodes) == 0:
            return
        node = nodes[0]
        cxml = node.canonicalize()
        digestValue = sha.sha(cxml).digest()
        digestExp = XPath.Compile('//ds:DigestValue')
        nodes = element.evaluate(expression=digestExp, processorNss={'ds':DSIG.BASE})
        receivedDigestValue = nodes[0].getValue()

        digestValue64 = base64.encodestring(digestValue)
        digestValueStripped = digestValue64.strip()
        # Hack to get rid of the newline at the end of digestValue64
        #if digestValue64[len(digestValue64) -1] == '\n':
        #    digestValueStripped = digestValue64[0:len(digestValue64)-1]
        
        if digestValueStripped != receivedDigestValue:
            raise VerifyError, 'DigestValues do not match'

        siExp = XPath.Compile('//ds:SignedInfo')
        nodes = element.evaluate(expression=siExp, processorNss={'ds':DSIG.BASE})
        signedInfo = nodes[0]
        signedInfoStr = signedInfo.canonicalize()
        sigExp = XPath.Compile('//ds:SignatureValue')
        nodes = element.evaluate(expression=sigExp,processorNss={'ds':DSIG.BASE})
        node = nodes[0]
        signatureStr = node.getValue()

        keyExp = XPath.Compile('//ds:KeyName')
        nodes = element.evaluate(expression=keyExp,processorNss={'ds':DSIG.BASE})
        node = nodes[0]

        self._contextID = node.getValue()
        home = GssSecurityContext.getResourceHome()
        secContext = home.get(self._contextID)

        verified = secContext.verifyMIC(signedInfoStr, base64.decodestring(signatureStr))
        if not verified:
            raise VerifyError, 'Signatures do not match'

        
    def sign(self, sw):
        context_id = self._contextID
        if context_id is None:
            raise RuntimeError, 'must set context to sign a message'

        #Find REFERENCES
        idExp = XPath.Compile('//*[@wsu:Id]')
        nodes = sw.dom.evaluate(expression=idExp, processorNss={'wsu':WSU.BASE})
        if len(nodes) == 0: 
            return
        node = nodes[0]
        URI = '#%s' %node.getAttributeValue(WSU.BASE, 'Id')

        # WS-Security 
        # SPECIFY SOAP-ENV:actor, move this to SoapWriter!!!
        pyobj = Holder()
        sig = Signature()
        pyobj._any = [sig]
        pyobj._attrs = {(SOAP.ENV,'actor'):""}

        sig._SignedInfo = Holder()
        sig._SignedInfo._CanonicalizationMethod = Holder()
        sig._SignedInfo._CanonicalizationMethod._attrs = {'Algorithm':self._can_algo}
        sig._SignedInfo._SignatureMethod = Holder()
        sig._SignedInfo._SignatureMethod._attrs = {'Algorithm':self._sig_algo}
        sig._SignedInfo._Reference = Holder()
        sig._SignedInfo._Reference._attrs = {'URI':URI}
        sig._SignedInfo._Reference._DigestMethod = Holder()
        sig._SignedInfo._Reference._DigestMethod._attrs = {'Algorithm':self._dig_algo}
        sig._SignedInfo._Reference._DigestValue = self.getDigestValue(sw)
        sig._SignatureValue = ''

        home = GssSecurityContext.getResourceHome()
        secContext = home.get(context_id)

        sig._KeyInfo = Holder()
        sig._KeyInfo._KeyName = context_id

        typecode = AUTH.schemas_xmlsoap_org_ws_2002_04_secext.Security_Dec()
        header_pyobjs = {(typecode.nspname,typecode.pname):pyobj}
        sw.serialize_header(pyobj, typecode)

        siExp = XPath.Compile('//ds:SignedInfo')
        nodes = sw.dom.evaluate(expression=siExp, processorNss={'ds':DSIG.BASE})
        signedInfo = nodes[0]
        signedInfoStr = signedInfo.canonicalize()

        sigExp = XPath.Compile('//ds:SignatureValue')
        nodes = sw.dom.evaluate(expression=sigExp, processorNss={'ds':DSIG.BASE})
        signatureValueNode = nodes[0]
        signatureValue = base64.encodestring(secContext.getMIC(signedInfoStr))
        signatureValueNode.createAppendTextNode(signatureValue)

    def getDigestValue(self, sw):
        '''
<ns1:createCounter xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ZSI="http://www.zolera.com/schemas/ZSI/" xmlns:ns1="http://counter.com" xmlns:ns2="http://schemas.xmlsoap.org/ws/2002/04/utility" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" ns2:Id="digestSource"></ns1:createCounter>
        '''
        digExp = XPath.Compile('//*[@wsu:Id="digestSource"]')
        nodes = sw.dom.evaluate(expression=digExp, processorNss={'wsu':WSU.BASE})
        node = nodes[0]
        cxml = node.canonicalize()
        digestValue = sha.sha(cxml).digest()
        return digestValue

